#include "testlib.h"
#include <iostream>
#include <vector>
#include <cassert>

using namespace std;

const int SIGMA = 'z' - 'a' + 1;
struct Mat{
    int n;
    vector<string> rep;
    Mat(int _n){
        n = _n;
        rep = vector<string>(n, string(n, '#'));
    }
    void setCell(int i, int j, char c){
        rep[i][j] = c;
    }
    char getCell(int i, int j){
        return rep[i][j];
    }
    void setInt(int i, int j, int val){
        assert(0<=val&&val<SIGMA);
        setCell(i, j, 'a'+val);
    }
    vector<string> getMat(){
        for(auto s: rep)
            for(auto c: s){
                assert('a'<=c&&c<='z');
            }
        return rep;
    }
    void printMat(){
        auto ret = getMat();
        for(auto t: ret){
        }
    }
};
Mat troll(int n){
    assert(n<=SIGMA);
    Mat mat(n);
    for(int i=0;i<n;i++)
        for(int j=0;j<n;j++)
            mat.setInt(i, j, (j-i+SIGMA)%SIGMA);
    int t1=0, t2=0;
    while(t1 == t2){
        t1 = rnd.next(n);
        t2 = rnd.next(n);
        mat.setCell(t1, n-1-t1, mat.getCell(t2, n-1-t2));
    }
    return mat;
}
Mat misleading(int n, int misleadingLength, int realWinLength, int misleadDist){
    Mat mat(n);
    assert(misleadingLength * 3 <= realWinLength);
    assert (misleadDist != misleadingLength/2);
    for(int i=0;i<n;i++)
        for(int j=0;j<n;j++)
            mat.setInt(i, j, rnd.next(SIGMA));
    for(int i=0;i<misleadingLength; i++){
        mat.setCell(0, i, 'a');
        mat.setCell(i, 0, 'a');
    }
    int start = misleadingLength/2;
    int l1 = misleadDist, c1 = start;
    int l2 = start, c2 = misleadDist;
    for(int i=0; i < realWinLength; i++){
        int val = rnd.next(SIGMA);
        mat.setInt(l1, c1, val);
        mat.setCell(l2, c2, mat.getCell(l1, c1));
        if(l1 + 1 == l2){
            c1 += 1;
            l2 += 1;
            continue;
        }
        if(l2 + 1 == l1){
            c2 += 1;
            c1 += 1;
            continue;
        }
        if(rnd.next(2)){
            l1 += 1;
            c2 += 1;
        } else {
            c1 += 1;
            l2 += 1;
        }
    }
    return mat;
}
Mat ambiguous(int n){
    assert(n>=SIGMA);
    Mat mat(n);
    for(int i=0;i<n;i++)
        for(int j=0;j<n;j++)
            mat.setCell(i, j, 'a');
    for(int i=0;i<SIGMA - 1;i++){
        mat.setInt(i, SIGMA - 2 -i, i+1);
    }
    int t1 = 0, t2 = 0;
    while(t1 == t2){
        t1 = rnd.next(SIGMA - 1);
        t2 = rnd.next(SIGMA - 1);
        mat.setCell(t1, (SIGMA - 2)-t1, mat.getCell(t2, (SIGMA - 2)-t2));
    }
    return mat;
}
Mat randomMat(int n){
    Mat mat(n);
    for(int i=0;i<n;i++)
        for(int j=0;j<n;j++)
            mat.setInt(i, j, rnd.next(SIGMA));
    return mat;
}
Mat strongRoadWithBreaks(int n, int breakFrequency){
    assert(n>60);
    Mat mat = randomMat(n);
    for(int i=0;i<SIGMA;i++)
        for(int j=0;j<SIGMA;j++){
            mat.setInt(i, j, (j-i+SIGMA)%SIGMA);
            mat.setInt(n-SIGMA+i,n-SIGMA+j, (j-i+SIGMA)%SIGMA);
        }
    int start = rnd.next(SIGMA);
    int startLine1 = start;
    int startCol1 = SIGMA;

    int startLine2 = SIGMA;
    int startCol2 = start;

    int endLine1 = n - startLine2;
    int endCol1 = n - startCol2;

    int endLine2 = n - startLine1;
    int endCol2 = n - startCol1;

    while(startLine1 != endLine1){
        int A = endLine1 - startLine1;
        int B = endCol1 - startCol1;
        if(rnd.next(breakFrequency)){
            mat.setCell(startLine1, startCol1, mat.getCell(startLine2, startCol2));
        }
        if(rnd.next(A+B)<A){
            startLine1+=1;
            startLine2+=1;
        } else {
            startCol1+=1;
            startCol2+=1;
        }
        assert(startLine1!=startLine2||startCol1!=startCol2);
    }
    return mat;
}
Mat sameHalf(int n, int startDiag, int len){
    assert(startDiag<=n);
    Mat mat = randomMat(n);
    int t=rnd.next(startDiag);
    int l1=t,c1=startDiag-t;
    int l2=t+1,c2=startDiag-t-1;
    for(int i=0; i < len; i++){
        mat.setCell(l1,c1,mat.getCell(l2, c2));
        if(rnd.next(2)) l1+=1,l2+=1;
        else c1+=1,c2+=1;
    }
    return mat;

}
void printTest(Mat mat){
    auto rep = mat.getMat();
    cout<<rep.size()<<'\n';
    for(auto s: rep){
        cout<<s<<'\n';
    }
}
int main(int argc, char** argv)
{
    registerGen(argc, argv, 1);
    string mode = opt<string>("mode");
    int n = opt<int>("n");
    if(mode == "troll") {
        printTest(troll(n));
    } else if(mode == "misleading") {
        int misleadingLength = opt<int>("misleadingLength");
        int realWinLength = opt<int>("realWinLength");
        int misleadDist = opt<int>("misleadDist");
        printTest(misleading(n, misleadingLength, realWinLength, misleadDist));
    } else if(mode == "ambiguous") {
        printTest(ambiguous(n));
    } else if(mode == "randomMat") {
        printTest(randomMat(n));
    } else if(mode == "strongRoadWithBreaks") {
        int breakFrequency = opt<int>("breakFrequency");
        printTest(strongRoadWithBreaks(n, breakFrequency));
    } else if(mode == "sameHalf") {
        int startDiag = opt<int>("startDiag");
        int len = opt<int>("len");
        printTest(sameHalf(n, startDiag, len));
    } else {
      assert(false);
    }

    return 0;
}
